package com.jobs.test;

import com.jobs.config.springConfig;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.api.BackgroundCallback;
import org.apache.curator.framework.api.CuratorEvent;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = springConfig.class)
public class zkTest {

    @Autowired
    private CuratorFramework client;

    //注意：
    //由于我在 zookeeper.properties 下配置的 namespace 是 jobs
    //因此下面对 zookeeper 的所有操作，默认情况下都是在 /jobs 节点下进行操作的

    @Test
    public void createTest() throws Exception {

        //默认情况下，创建的都是持久化的节点，一旦创建，永久存储

        //在创建节点时，没有指定存储的数据，默认情况下存储的是当前客户端机器的 ip 地址
        String path1 = client.create().forPath("/test1");
        System.out.println("成功创建节点：" + path1);

        //获取节点中存储的数据，由于在创建时没有存储数据，所以发现存储的是当前客户端机器的 ip 地址
        byte[] data1 = client.getData().forPath("/test1");
        System.out.println("存储的数据为：" + new String(data1));

        System.out.println("-------------------------------------");

        //创建节点时，同时存储数据
        String path2 = client.create().forPath("/test2", "乔京飞".getBytes());
        System.out.println("成功创建节点：" + path2);

        //获取节点中存储的数据
        byte[] data2 = client.getData().forPath("/test2");
        System.out.println("存储的数据为：" + new String(data2));
    }

    @Test
    public void createTempTest() throws Exception {

        //创建临时节点，一旦当前连接断开，zookeeper 会自动销毁所创建的节点

        String path = client.create().withMode(CreateMode.EPHEMERAL).forPath("/test3", "任肥肥".getBytes());
        System.out.println("成功创建临时节点：" + path);
        byte[] data = client.getData().forPath("/test3");
        System.out.println("存储的数据为：" + new String(data));

        //注意：
        //由于当前方法执行完后，zookeeper 连接就断开了，临时节点就销毁了
        //所以你可以通过命令行的方式，查看 zookeeper 中 /jobs 下的节点，发现找不到 /jobs/test3 节点
        //但是：通过上面运行的程序，打印出节点信息和节点存储的数据，可以断定：/jobs/test3 节点曾经被创建过
    }

    @Test
    public void autoCreateParentsNodeTest() throws Exception {

        //当我们创建深层次节点时，可能路径上的父节点并不存在，所以仅仅 create 无法同时创建父子节点
        //可以使用 creatingParentsIfNeeded 方法，同时创建父子节点

        String path = client.create().creatingParentsIfNeeded().forPath("/test4/node1");
        System.out.println(path);
    }

    //----------------------------------------
    //----------------------------------------

    @Test
    public void getTest() throws Exception {

        //查询节点中存储的数据
        byte[] data = client.getData().forPath("/test4/node1");
        //由于在创建节点时，没有存储数据，所以默认存储的是当前客户端所在机器的 ip 地址
        System.out.println(new String(data));
    }

    @Test
    public void getChildrenNodeTest() throws Exception {

        //获取一个节点下面，所有的子节点，并打印出来
        //注意：由于 namespace 配置的是 jobs ，所以这里的 / 代表的是 /jobs
        List<String> pathlist = client.getChildren().forPath("/");
        for (String path : pathlist) {
            System.out.println(path);
        }
    }

    @Test
    public void getNodeStatusTest() throws Exception {

        //查看一个节点的详细状态信息，相当于执行命令 ls -s 节点

        Stat status = new Stat();
        client.getData().storingStatIn(status).forPath("/test2");

        //由于开发 Curator 的程序员太懒惰了，
        //所以打印出来的每个数字的含义，请查看 Stat 类的 toString 方法
        System.out.println(status);
    }

    //----------------------------------------
    //----------------------------------------

    @Test
    public void setDataTest() throws Exception {

        byte[] data1 = client.getData().forPath("/test1");
        System.out.println("【修改前】存储的数据为：" + new String(data1));

        //修改数据为当前时间毫秒值
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowTime = sdf.format(new Date());
        client.setData().forPath("/test1", nowTime.getBytes());

        byte[] data2 = client.getData().forPath("/test1");
        System.out.println("【修改后】存储的数据为：" + new String(data2));
    }

    @Test
    public void SetDataTest2() throws Exception {

        //如果所要修改的数据，是公共资源，存在并发修改的情况
        //为了确保修改数据的安全性，可以采用版本号进行控制修改操作
        //也就是所谓的乐观锁机制
        //在修改数据时，如果发现数据版本号不是自己读取的版本号，则放弃修改

        Stat status = new Stat();
        client.getData().storingStatIn(status).forPath("/test2");
        //获取数据版本号
        int version1 = status.getVersion();
        System.out.println("【修改前】的数据版本号：" + version1);

        //要修改的节点版本号，与所提供的版本号，不一致时，会报异常
        client.setData().withVersion(version1).forPath("/test2", "任肥肥".getBytes());

        client.getData().storingStatIn(status).forPath("/test2");
        //获取数据版本号
        int version2 = status.getVersion();
        System.out.println("【修改后】的数据版本号：" + version2);
    }

    //----------------------------------------
    //----------------------------------------

    @Test
    public void deleteTest() throws Exception {

        //删除节点，如果节点下有子节点的话，无法删除
        //要删除的节点不存在时，会报异常
        client.delete().forPath("/test1");
    }

    @Test
    public void deleteAllTest() throws Exception {

        //删除节点，即使有子节点，连同子节点一起删除
        //要删除的节点不存在时，会报异常
        client.delete().deletingChildrenIfNeeded().forPath("/test4");
    }

    @Test
    public void testDelete3() throws Exception {

        //保证删除成功，如果网络有问题，则会定期重试
        //要删除的节点不存在时，会报异常
        client.delete().guaranteed().forPath("/test2");
    }

    @Test
    public void testDelete4() throws Exception {

        //节点删除成功后，执行回调方法
        //要删除的节点不存在时，会报异常

        client.delete().guaranteed().inBackground((client, event) -> {
            System.out.println("Lambda表达式写法：节点删除操作执行成功....");
            System.out.println(event);
        }).forPath("/test1");


        client.delete().guaranteed().inBackground(new BackgroundCallback() {
            @Override
            public void processResult(CuratorFramework client, CuratorEvent event) throws Exception {

                //参数含义：
                //client 其实就是客户端连接对象，可以对 zookeeper 节点进行增删改查
                //event 事件状态对象

                System.out.println("匿名函数写法：节点删除操作执行成功....");
                System.out.println(event);
            }
        }).forPath("/test2");
    }
}
